program conred32;
{$F+}
{
***********************************************************
"Puente de tiristores"
Autores: JRomero - Arodri
Fecha: 18-04-98

		 La simulacin corresponde a un puente de tiristores
		 con iL=Id , y se obeservarn los siguientes
		 parmetros:

		 * tensiones de entrada (lineas RST), contrastada con la de salida
		  tomando como carga una fuente de corriente.
		 * tensin sobre el tiristor 1.
		 * corrientes de cada tiristor.

		 Se podr afectar el ngulo de disparo  y
		 el valor del mismo estar acotado entre 0 y 170 grados. El
		 recorrido se har a pasos de 5 grados.

		 OBS: para implementar el disparo por el control arcocoseno
		 se toma como referencia a la fase R y se supone que esta es
		 Vp sin(w*t).
**********************************************************
}

uses
wincrt, xmatdefs, FilUSR, UCntrl, Simul4, traxp, variable,
lib_basi, libep02, libep, libdig01, memorias,dibus;

const w=PI*100;           {frecuencia de red=50 hz}
		sexrad=PI/180;      {constante de conversin de sexagesimal a rad}
		periodo=20e-3;      {periodo=2*PI/w}
		grillay=10;          {divisiones de la grilla en y}
		grillax=12;			  {divisiones de la grilla en x}
		nper=1;				  {nmero de periodos a visualizar   (nper>=1)}
		rshunt=0.001;		  {valor de resitencias de medicin de corriente}
		NumeroPuntos=512;		{puntos de muestreo}

var   wt,alpha:NReal;
		{ variables definidas para muestrear UD}
		UD,
		tMuestra,
		NMuestra:Nreal;
		mem1:Tmemoria;

(*********************************************************************
  Funcin para el disparo del tiristor.
*********************************************************************)
{
	El parmetro ALPHA es tomado como control del disparo del tiristor.
}

function disparo(k:integer): boolean;
var vref:NREAL;
begin
	vref:=-cos(alpha*sexrad);
	disparo:=(vref <= sin(w*t - 2*PI/3 - (k-1) * PI/3)) and (cos(w*t - 2*PI/3 - (k-1) * PI/3)>=0);
{	DISPARO:=TRUE;}
end;

(***********************************************************************
	Funciones de definicin del circuito

	Cirt1: componentes no booleanos
	Cirt2: componentes booleanos
***********************************************************************)

procedure cirt1;
begin
vg(1,11,1,311);							{generador R de 220 Vrms}
vg(2,11,2,311);							{generador T de 220 Vrms}
vg(3,11,3,311);							{generador S de 220 Vrms}


{resistencias de medicin de corrientes}
r(1,4,rshunt);
r(2,5,rshunt);
r(3,6,rshunt);
r(1,8,rshunt);
r(2,9,rshunt);
r(3,10,rshunt);

ig(0,7,0,10);								{fuente de corriente Id=10 A}

end;

procedure cirt2;
begin
thmi(4,7,disparo(1));							{Tiristor 1}
thmi(0,10,disparo(2));							{Tiristor 2}
thmi(5,7,disparo(3));							{Tiristor 3}
thmi(0,8,disparo(4));							{Tiristor 4}
thmi(6,7,disparo(5));							{Tiristor 5}
thmi(0,9,disparo(6));							{Tiristor 6}
end;


(************************************************************************
	Definicin de la seal del generador VG[1]
************************************************************************)
procedure funcsr(t:NReal);
begin
rf[1]:=sin(w*t);				{R}
rf[2]:=sin(w*t-2*PI/3);    {S}
rf[3]:=sin(w*t-4*PI/3);    {T}
end;




(************************************************************************
	Procedimientos necesarios de Simeep:

	inicio:	inicializacin de simulaciones, def. de pantalla,etc
	pesca: 	captura e impresin en pantalla de variables observadas
	fin:     procedimiento de finalizacin

************************************************************************)

procedure inicio;
begin
	nn:=11;					{nmero de nodos, sin incluir tierra (0)}
	nindep:=3;     		{nmero de funciones definidas en funcsr = numero de generadores}
	tinicial:=0;			{tiempo inicial de la simulacin}
	tfinal:=nper*periodo;	{tiempo final de la simulacin}


	{parmetros de simulacin}
	dtmax0:=5e-6;
	dtmin:=dtmax0/100;
	PasoDivisor:=6;
	TmaxTmin:=1e3;

	{inicializacin de la unidad de grficos}
	traxp.tinicial:=tinicial;
	traxp.tfinal:=tfinal;

	{inicializacin del modo grfico}
	iniciegr;

	{grillas en pantalla}
	gridx:=grillax;
	gridy:=grillay;

	{matriz de visualizacin de 1 por 4}
	subplot(1,3);


	{primera pantalla}
	definaY(0,-750,150);
	titulo('uD(t) (amarillo) y tensiones de linea: +-RS(rojo),+-ST(negro),+-TR(azul)');
	xlabel('30/DIV');
	ylabel('150 V/DIV');
	linea0;
	grid;

	gridy:=13;
	{segunda pantalla}
	definaY(1,0,10);
	titulo('corriente de tiristores');
	xlabel('30/DIV');
	ylabel('  6  5  4  3   2  1');
	grid;

	gridy:=grillay;

	{tercera pantalla}
	definaY(2,-750,150);
	titulo('uAK1(amarillo) y tensiones de lnea');
	xlabel('30/DIV');
	ylabel('150 V/DIV');
	linea0;
	grid;

	{canal 4,5,6 en la misma pantalla que el canal 0}
	superponga(3,0);
	superponga(4,0);
	superponga(5,0);
	superponga(11,0);
	superponga(12,0);
	superponga(13,0);

	{canal 7,8,9,10,11,12 en la misma pantalla que el canal 1}
	superponga(6,1);
	superponga(7,1);
	superponga(8,1);
	superponga(9,1);
	superponga(10,1);

	{canal 7,8,9,10,11,12 en la misma pantalla que el canal 2}
	superponga(14,2);
	superponga(15,2);
	superponga(16,2);
	superponga(17,2);
	superponga(18,2);
	superponga(19,2);

	{colores a mostrar por canal}

	{primera pantalla}
	definacolor(0,amarillo);			{V_salida}
	definacolor(3,rojo);			{R}
	definacolor(4,negro);      {S}
	definacolor(5,azul);       {T}
	definacolor(11,rojo);			{R}
	definacolor(12,negro);      {S}
	definacolor(13,azul);       {T}

	{segunda pantalla}
	definacolor(1,rojo);		{iT1}
	definacolor(6,8);			{iT2}
	definacolor(7,8);			{iT3}
	definacolor(8,8);		{iT4}
	definacolor(9,8);		{iT5}
	definacolor(10,8);		{iT6}

	{tercera pantalla}
	definacolor(2,amarillo);			{uAK1}
	definacolor(14,rojo);			{}
	definacolor(15,negro);		{}
	definacolor(16,azul);			{}
	definacolor(17,rojo);			{}
	definacolor(18,negro);		{}
	definacolor(19,azul);			{}

end;

{____________________________________________________________________}

procedure fin;
begin
TermineGr;
halt;
end;

{____________________________________________________________________}

procedure pesca;

var   k,enx:integer;
		grados:NReal;
begin

	traxp.t:=t;					{indicamos a traxp el tiempo de simulacin}
	wt:=w/sexrad*t;

	{*etiquetas de grficas*}
	grados:=(w/sexrad)*t;
	enX:=PUNTOX+21; {paso la ordenada x que pinta la onda a la variable enX por medio de la funcion puntox}
	{ANOTAR_EVENTO(numeracin, GRAD1,GRAD2,GRAD, ENX, y,y1a,y1b,y2a,y2b,etiqueta)}
	{alpha}ANOTAR_EVENTO(1,30,30+alpha,grados,enx,70,20,140,20,140, CHR(224)+' para T1');

	{etiquetas}
	ANOTAR_TEXTO(1,6,grados,enx,15, negro,'uTS');
	ANOTAR_TEXTO(2,66,grados,enx,15, rojo,'uRS');
	ANOTAR_TEXTO(3,126,grados,enx,15, azul,'uRT');
	ANOTAR_TEXTO(4,186,grados,enx,15, negro,'uST');
	ANOTAR_TEXTO(5,246,grados,enx,15, rojo,'uSR');
	ANOTAR_TEXTO(6,300,grados,enx,15, azul,'uTR');

	ANOTAR_TEXTO(7,6,grados,enx,335, negro,'uTS');
	ANOTAR_TEXTO(8,66,grados,enx,335, rojo,'uRS');
	ANOTAR_TEXTO(9,126,grados,enx,335, azul,'uRT');
	ANOTAR_TEXTO(10,186,grados,enx,335, negro,'uST');
	ANOTAR_TEXTO(11,246,grados,enx,335, rojo,'uSR');
	ANOTAR_TEXTO(12,300,grados,enx,335, azul,'uTR');

	{etiquetas de conmutacion}
	ANOTAR_TEXTO(13,20+alpha,grados,enx,130, amarillo,'5->1');
	ANOTAR_TEXTO(14,80+alpha,grados,enx,130, amarillo,'6->2');
	ANOTAR_TEXTO(15,140+alpha,grados,enx,130, amarillo,'1->3');
	if (200+alpha)<345 then
		ANOTAR_TEXTO(16,200+alpha,grados,enx,130, amarillo,'2->4')
		else if (200+alpha)>360 then
		ANOTAR_TEXTO(16,200-360+alpha,grados,enx,130, amarillo,'2->4');
	if (260+alpha)<345 then
		ANOTAR_TEXTO(17,260+alpha,grados,enx,130, amarillo,'3->5')
		else if (260+alpha)>360 then
		ANOTAR_TEXTO(17,260-360+alpha,grados,enx,130, amarillo,'3->5');
	if (320+alpha)<345 then
		ANOTAR_TEXTO(18,320+alpha,grados,enx,130, amarillo,'4->6')
		else if (320+alpha)>360 then
		ANOTAR_TEXTO(18,320-360+alpha,grados,enx,130, amarillo,'4->6');

	{captura de valores}

	{primera pantalla}
	trazo(3,v[1]-v[2]);			{R}
	trazo(4,v[2]-v[3]);      {S}
	trazo(5,v[3]-v[1]);       {T}
	trazo(11,v[2]-v[1]);			{R}
	trazo(12,v[3]-v[2]);      {S}
	trazo(13,v[1]-v[3]);       {T}
	trazo(0,v[7]+20);			{V_salida}
	trazo(0,v[7]-20);			{V_salida}

	{segunda pantalla}
	trazo(1,(v[1]-v[4])*(1/rshunt)+110);		{iT1}
	trazo(6,(v[10]-v[3])*(1/rshunt)+90);			{iT2}
	trazo(7,(v[2]-v[5])*(1/rshunt)+70);			{iT3}
	trazo(8,(v[8]-v[1])*(1/rshunt)+50);		{iT4}
	trazo(9,(v[3]-v[6])*(1/rshunt)+30);		{iT5}
	trazo(10,(v[9]-v[2])*(1/rshunt)+10);		{iT6}

	trazo(1,110);		{iT1}
	trazo(6,90);			{iT2}
	trazo(7,70);			{iT3}
	trazo(8,50);		{iT4}
	trazo(9,30);		{iT5}
	trazo(10,10);		{iT6}


	{tercera pantalla}
	trazo(14,v[1]-v[2]);			{R}
	trazo(15,v[2]-v[3]);      {S}
	trazo(16,v[3]-v[1]);       {T}
	trazo(17,v[2]-v[1]);			{R}
	trazo(18,v[3]-v[2]);      {S}
	trazo(19,v[1]-v[3]);       {T}

	trazo(2,v[1]-v[7]+5);			{uAK1}
	trazo(2,v[1]-v[7]-5);			{uAK1}


	{pesca de UD para muestreo}
	if tMuestra>=periodo/6 then
	begin
		tMuestra:=0;
		NMuestra:=NMuestra+1;

		mem1.fft;
		ud:=mem1.ValorMedio;
		mem1.retrocesobarrido(periodo/6);
		if (ud<2) and (ud>-2) then ud:=0; {truco}
	end
	else tMuestra:=t-NMuestra*periodo/6;
	mem1.tic(tMuestra,v[7]);

	if t >= tfinal then
	begin
	{llegamos al final de la simulacin}

		{para el muestreo de UD}
		tMuestra:=0;
		NMuestra:=0;

		for k:= 0 to 19 do
		begin
			borrecanal(k,0);
			desligue(k);
			borde;

			if (k<>1) and (k<>6) and (k<>7) and (k<>8) and (k<>9) and (k<>10) then
			begin
				grid;
				linea0;
			end
			else
			begin
				gridy:=13;
				grid;
				gridy:=grillay;
			end;
		end;

		{nueva simulacin}
		t:= t-tfinal+tinicial;


	end;



end;


(*********************************************************************
 Principal
*********************************************************************)
BEGIN
	quita:=5;			{separaciones de pantallas}

	{para el muestreo de UD}
	UD:=446;	{valor "inicial", antes de hacer el primer calculo}
	NMuestra:=0;
	tMuestra:=0;
	mem1:= TMemoria.Create_init(periodo/6/numeropuntos,numeropuntos,0,0);


	{decimos a simeep donde estan nuestras funciones}
	FilUSR.Cirt1:=Cirt1;
	FilUSR.Cirt2:=Cirt2;
	FilUSR.FuncsR:=FuncsR;
	FilUSR.Inicio:=Inicio;
	FilUSR.Pesca:=Pesca;
	FilUSR.Fin:=Fin;

	{definicin de controles}
	UCntrl.init;

	UCntrl.insertdisplay(new(PCtrlVerTexto, init
	{ rnglon, texto1  ,            texto2,             texto3}
	(	0,	'Puente de tiristores','con iL=Id.','')));


	UCntrl.insert(new(PCtrlBotonVar,Init
	{renglon, nombre,   unidad,decimales, min, max,   def, inc,    var}
	(  8,   ' Disparo',  ''  ,0        ,   0, 170,     30,  5,  alpha)));

	UCntrl.insertdisplay(new(PCtrlVer2Var,init
	{ renglon, nombre1, uni1,deci1,var1 ,nombre2, uni2,deci2,var2}
	(       7, '  Ud',  'V',  0  ,ud ,' grados', '',  1  ,wt)));

	paseo;												  {simulacin}

	mem1.Free;

end.
